# Python 日期时间

> 原文： [https://www.programiz.com/python-programming/datetime](https://www.programiz.com/python-programming/datetime)

#### 在本文中，您将借助示例学习使用 Python 操作日期和时间。

Python 有一个名为`datetime`的模块，用于处理日期和时间。 在深入研究之前，让我们创建一些与日期和时间有关的简单程序。

* * *

### 示例 1：获取当前日期和时间

```py
import datetime

datetime_object = datetime.datetime.now()
print(datetime_object) 
```

当您运行程序时，输出将类似于：

```py
2018-12-19 09:26:03.478039
```

在这里，我们使用`import datetime`语句导入了**日期时间**模块。

`datetime`模块中定义的类别之一是`datetime`类别。 然后，我们使用`now()`方法创建一个包含当前本地日期和时间的`datetime`对象。

* * *

### 示例 2：获取当前日期

```py
 import datetime

date_object = datetime.date.today()
print(date_object) 
```

When you run the program, the output will be something like:

```py
2018-12-19
```

在此程序中，我们使用了`date`类中定义的`today()`方法来获取包含当前本地日期的`date`对象。

* * *

**日期时间里面是什么？**

我们可以使用[`dir()`](/python-programming/methods/built-in/dir)函数来获取包含模块所有属性的列表。

```py
import datetime

print(dir(datetime))
```

运行该程序时，输出为：

```py
 ['MAXYEAR', 'MINYEAR', '__builtins__', '__cached__', '__doc__', '__file__', '__loader__', '__name__', '__package__', '__spec__', '_divide_and_round', 'date', 'datetime', 'datetime_CAPI', 'time', 'timedelta', 'timezone', 'tzinfo'] 
```

* * *

`datetime`模块中常用的类是：

*   日期类
*   时间类
*   日期时间类
*   时间差异类

* * *

## `datetime.date`类

您可以从`date`类实例化`date`对象。 日期对象代表日期（年，月和日）。

* * *

### 示例 3：代表日期的`Date`对象

```py
 import datetime

d = datetime.date(2019, 4, 13)
print(d) 
```

当你运行程序时，输出将是：

```py
2019-04-13
```

如果您想知道，以上示例中的`date()`是`date`类的构造器。 构造器采用三个参数：年，月和日。

变量`d`是`date`对象。

* * *

我们只能从`datetime`模块导入`date`类。 这是如何做：

```py
 from datetime import date

a = date(2019, 4, 13)
print(a)
```

* * *

### 示例 4：获取当前日期

您可以使用名为`today()`的类方法来创建包含当前日期的`date`对象。 这是如何做：

```py
 from datetime import date

today = date.today()

print("Current date =", today) 
```

* * *

### 示例 5：从时间戳获取日期

我们还可以根据时间戳创建`date`对象。 Unix 时间戳是 UTC 特定日期到 1970 年 1 月 1 日之间的秒数。 您可以使用`fromtimestamp()`方法将时间戳转换为日期。

```py
 from datetime import date

timestamp = date.fromtimestamp(1326244364)
print("Date =", timestamp) 
```

当你运行程序时，输出将是：

```py
Date = 2012-01-11
```

* * *

### 示例 6：打印今天的年，月和日

我们可以轻松地从日期对象获取年，月，日，星期几等。 这是如何做：

```py
 from datetime import date

# date object of today's date
today = date.today() 

print("Current year:", today.year)
print("Current month:", today.month)
print("Current day:", today.day) 
```

* * *

## datetime.time

从`time`类实例化的时间对象表示本地时间。

* * *

### 示例 7：代表时间的时间对象

```py
 from datetime import time

# time(hour = 0, minute = 0, second = 0)
a = time()
print("a =", a)

# time(hour, minute and second)
b = time(11, 34, 56)
print("b =", b)

# time(hour, minute and second)
c = time(hour = 11, minute = 34, second = 56)
print("c =", c)

# time(hour, minute, second, microsecond)
d = time(11, 34, 56, 234566)
print("d =", d) 
```

当你运行程序时，输出将是：

```py
 a = 00:00:00
b = 11:34:56
c = 11:34:56
d = 11:34:56.234566 
```

* * *

### 示例 8：打印小时，分钟，秒和微秒

创建`time`对象后，可以轻松打印其属性，例如`hour`，`minute`等。

```py
 from datetime import time

a = time(11, 34, 56)

print("hour =", a.hour)
print("minute =", a.minute)
print("second =", a.second)
print("microsecond =", a.microsecond) 
```

当您运行示例时，输出将是：

```py
 hour = 11
minute = 34
second = 56
microsecond = 0 
```

请注意，我们尚未传递`microsecond`参数。 因此，将打印其默认值`0`。

* * *

## datetime.datetime

`datetime`模块具有一个名为`dateclass`的类，可以包含来自**日期**和**时间**对象的信息。

* * *

### 示例 9：Python `datetime`对象

```py
 from datetime import datetime

#datetime(year, month, day)
a = datetime(2018, 11, 28)
print(a)

# datetime(year, month, day, hour, minute, second, microsecond)
b = datetime(2017, 11, 28, 23, 55, 59, 342380)
print(b) 
```

当你运行程序时，输出将是：

```py
 2018-11-28 00:00:00
2017-11-28 23:55:59.342380 
```

`datetime()`构造器中的前三个参数`year`，`month`和`day`是必填项。

* * *

### 示例 10：打印年，月，时，分和时间戳

```py
 from datetime import datetime

a = datetime(2017, 11, 28, 23, 55, 59, 342380)
print("year =", a.year)
print("month =", a.month)
print("hour =", a.hour)
print("minute =", a.minute)
print("timestamp =", a.timestamp()) 
```

当你运行程序时，输出将是：

```py
 year = 2017
month = 11
day = 28
hour = 23
minute = 55
timestamp = 1511913359.34238 
```

* * *

## datetime.timedelta

`timedelta`对象表示两个日期或时间之间的时差。

* * *

### 示例 11：两个日期和时间之间的时差

```py
 from datetime import datetime, date

t1 = date(year = 2018, month = 7, day = 12)
t2 = date(year = 2017, month = 12, day = 23)
t3 = t1 - t2
print("t3 =", t3)

t4 = datetime(year = 2018, month = 7, day = 12, hour = 7, minute = 9, second = 33)
t5 = datetime(year = 2019, month = 6, day = 10, hour = 5, minute = 55, second = 13)
t6 = t4 - t5
print("t6 =", t6)

print("type of t3 =", type(t3)) 
print("type of t6 =", type(t6)) 
```

当你运行程序时，输出将是：

```py
 t3 = 201 days, 0:00:00
t6 = -333 days, 1:14:20
type of t3 = <class 'datetime.timedelta'>
type of t6 = <class 'datetime.timedelta'> 
```

注意，`t3`和`t6`均为`<class 'datetime.timedelta'>`类型。

* * *

### 示例 12：两个`timedelta`对象之间的差异

```py
 from datetime import timedelta

t1 = timedelta(weeks = 2, days = 5, hours = 1, seconds = 33)
t2 = timedelta(days = 4, hours = 11, minutes = 4, seconds = 54)
t3 = t1 - t2

print("t3 =", t3) 
```

当你运行程序时，输出将是：

```py
 t3 = 14 days, 13:55:39 
```

在这里，我们创建了两个`timedelta`对象`t1`和`t2`，它们的区别被打印在屏幕上。

* * *

### 示例 13：打印负`timedelta`对象

```py
 from datetime import timedelta

t1 = timedelta(seconds = 33)
t2 = timedelta(seconds = 54)
t3 = t1 - t2

print("t3 =", t3)
print("t3 =", abs(t3)) 
```

当你运行程序时，输出将是：

```py
 t3 = -1 day, 23:59:39
t3 = 0:00:21 
```

* * *

### 示例 14：持续时间（以秒为单位）

您可以使用`total_seconds()`方法获得`timedelta`对象中的总秒数。

```py
 from datetime import timedelta

t = timedelta(days = 5, hours = 1, seconds = 33, microseconds = 233423)
print("total seconds =", t.total_seconds()) 
```

当你运行程序时，输出将是：

```py
 total seconds = 435633.233423 
```

* * *

您还可以使用`+`运算符找到两个日期和时间的总和。 另外，您可以将`timedelta`对象乘以整数和浮点数。

* * *

## Python 格式的日期时间

日期和时间的表示方式在不同的地方，组织等中可能有所不同。在美国，使用`mm/dd/yyyy`更为常见，而在英国，`dd/mm/yyyy`更为常见。

Python 有`strftime()`和`strptime()`方法来处理。

* * *

### Python `strftime()` - 日期时间字符串对象

`strftime()`方法在类`date`，`datetime`和`time`下定义。 该方法从给定的`date`，`datetime`或`time`对象创建格式化的字符串。

* * *

### 示例 15：使用`strftime()`格式化日期

```py
 from datetime import datetime

# current date and time
now = datetime.now()

t = now.strftime("%H:%M:%S")
print("time:", t)

s1 = now.strftime("%m/%d/%Y, %H:%M:%S")
# mm/dd/YY H:M:S format
print("s1:", s1)

s2 = now.strftime("%d/%m/%Y, %H:%M:%S")
# dd/mm/YY H:M:S format
print("s2:", s2) 
```

When you run the program, the output will be something like:

```py
 time: 04:34:52
s1: 12/26/2018, 04:34:52
s2: 26/12/2018, 04:34:52 
```

这里，`%Y`，`%m`，`%d`，`%H`等是格式代码。`strftime()`方法采用一个或多个格式代码，并根据该代码返回格式化的字符串。

在上述程序中，`t`，`s1`和`s2`是字符串。

*   `%Y` - 年`0001, ..., 2018, 2019, ..., 9999`
*   `%m` - 月`01, 02, ..., 11, 12`
*   `%d` - 天`01, 02, ..., 30, 31`
*   `%H` - 小时`00, 01, ..., 22, 23`
*   `%M` - 分钟`00, 01, ..., 58, 59`
*   `%S` - 秒`00, 01, ..., 58, 59`

要了解有关`strftime()`和格式代码的更多信息，请访问： [Python `strftime()`](/python-programming/datetime/strftime)。

* * *

### Python `strptime()` - 日期时间的字符串

`strptime()`方法根据给定的字符串（代表日期和时间）创建`datetime`对象。

* * *

### 示例 16：`strptime()`

```py
 from datetime import datetime

date_string = "21 June, 2018"
print("date_string =", date_string)

date_object = datetime.strptime(date_string, "%d %B, %Y")
print("date_object =", date_object) 
```

当你运行程序时，输出将是：

```py
 date_string = 21 June, 2018
date_object = 2018-06-21 00:00:00 
```

`strptime()`方法采用两个参数：

1.  代表日期和时间的字符串
2.  等同于第一个参数的格式代码

顺便说一下，`%d`，`%B`和`%Y`格式代码分别用于`day`，`month`（全名）和`year`。

访问 [Python `strptime()`](/python-programming/datetime/strptime)了解更多信息。

* * *

## 使用 Python 处理时区

假设您正在一个项目上，需要根据其时区显示日期和时间。 建议您使用第三方[`pytz`模块](http://pytz.sourceforge.net/)，不要自己处理时区。

```py
 from datetime import datetime
import pytz

local = datetime.now()
print("Local:", local.strftime("%m/%d/%Y, %H:%M:%S"))

tz_NY = pytz.timezone('America/New_York') 
datetime_NY = datetime.now(tz_NY)
print("NY:", datetime_NY.strftime("%m/%d/%Y, %H:%M:%S"))

tz_London = pytz.timezone('Europe/London')
datetime_London = datetime.now(tz_London)
print("London:", datetime_London.strftime("%m/%d/%Y, %H:%M:%S")) 
```

When you run the program, the output will be something like:

```py
 Local time: 2018-12-20 13:10:44.260462
America/New_York time: 2018-12-20 13:10:44.260462
Europe/London time: 2018-12-20 13:10:44.260462 
```

在这里，`datetime_NY`和`datetime_London`是包含其各自时区的当前日期和时间的`datetime`对象。